/**
 * @file ysocket.cc
 * @brief TCP/IPå ()
 *
 * @author Yashiro Takeshi <yashiromann@nifty.com> 
 * @date 2004-02-01
 *
 * Copyright (C) 2004, Yashiro Takeshi
 */

#include <btron/libapp.h>
#include <bstring.h>
#include <bstdio.h>

#include "ytcpsocket.h"
#include "yexception.h"

namespace YLibDraft {

YTCPSocket::YTCPSocket(W mode = SOCK_STREAM)
{
  sock_id = so_socket(PF_INET, mode, IPPROTO_TCP);
  if (sock_id < 0){
    YString str((TC[]){L"åȤ˼Ԥޤ"});
    throw new YException(str, sock_id);
  }
}

YTCPSocket::YTCPSocket(W sockfd, W mode = SOCK_STREAM)
{
  sock_id = sockfd;
}

void YTCPSocket::bind(unsigned short port)
{
  W err = 0;
  if (sock_id < 0){
    YString str((TC[]){L"åȤƤޤ"});
    throw new YException(str, sock_id);
  }
  bzero(&server_addr, sizeof(server_addr));
  server_addr.sin_family = AF_INET;
  server_addr.sin_port = htons(port);
  server_addr.sin_addr.s_addr = INADDR_ANY;

  err = so_bind(sock_id, (sockaddr*)&server_addr, sizeof(server_addr));
}

void YTCPSocket::listen(int max)
{
  W err = 0;
  if (sock_id < 0){
    YString str((TC[]){L"åȤƤޤ"});
    throw new YException(str, sock_id);
  }
  else {
    err = so_listen(sock_id, max);
  }
}

YTCPSocket* YTCPSocket::accept()
{
  sockaddr addr;
  W nlen = sizeof(server_addr);

  W err = so_accept(sock_id, &addr, &nlen);
  if (err < 0){
    return 0;
  }
  
  return new YTCPSocket(err, SOCK_STREAM);
}

void YTCPSocket::connect(const char* hostname, unsigned short port)
{
  char addr[5], tmp[256], buf[HBUFLEN];
  W err = 0;
  
  bzero(reinterpret_cast<B*>(&server_addr), sizeof(sockaddr_in));
  server_addr.sin_family = PF_INET;
  server_addr.sin_port = htons(port);
  
  // get host
  if (inet_pton(AF_INET, hostname, addr) == 1){
    //err = so_gethostbyaddr(addr, 4, AF_INET, &host_info, buf);
    bcopy(reinterpret_cast<B*>(addr),
          reinterpret_cast<B*>(&(server_addr.sin_addr)), 4);
  }
  else {
    err = so_gethostbyname(const_cast<char*>(hostname), &host_info, buf);
    if (err < 0){
      //printf("%d\n", err >> 16);
      YString str((TC[]){L"ۥȤĤޤ"});
      throw new YException(str, err);
    }
    bcopy(reinterpret_cast<B*>(host_info.h_addr),
          reinterpret_cast<B*>(&(server_addr.sin_addr)), host_info.h_length);
  }

  // connect

  err = so_connect(sock_id, (sockaddr *)&server_addr, sizeof(sockaddr_in));  
  if (err < 0){
    YString str((TC[]){L"³˼Ԥޤ"});
    throw new YException(str, sock_id);
  }
}

void YTCPSocket::read(B* p, int nbytes){
  for (int i = 0; i < nbytes; ){
    W err = so_read(sock_id, (B*)(p + i), nbytes - i);
    if (err <= 0){
      YString str((TC[]){L"åȤɤ߹ߤ˼Ԥޤ"});
      //printf("%d\n", err >> 16);
      throw new YException(str, err);
    }
    i += err;
  }
}
  
void YTCPSocket::write(const B* p, int nbytes){
  for (int i = 0; i < nbytes; ){
    W err = so_write(sock_id, const_cast<B*>(p + i), nbytes);
    if (err < 0){
      YString str((TC[]){L"åȤؤν񤭹ߤ˼Ԥޤ"});
      throw new YException(str, err);
    }
    i += err;
  }
}

}
